==> libcsv-2.0.1.tar.gz: libcsv-2.0.1/examples/csvinfo.c <==
/*
csvinfo - reads CSV data from input file(s) and reports the number
          of fields and rows encountered in each file

Copyright (C) 2007  Robert Gamble

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include "libcsv/csv.h"

struct counts {
  long unsigned fields;
  long unsigned rows;
};

void cb1 (char *s, size_t len, void *data) { ((struct counts *)data)->fields++; }
void cb2 (char c, void *data) { ((struct counts *)data)->rows++; }

static int is_space(char c) {
  if (c == CSV_SPACE || c == CSV_TAB) return 1;
  return 0;
}

static int is_term(char c) {
  if (c == CSV_CR || c == CSV_LF) return 1;
  return 0;
}


int
main (int argc, char *argv[])
{
  FILE *fp;
  struct csv_parser *p;
  char buf[1024];
  size_t bytes_read;
  unsigned char options = 0;
  struct counts c = {0, 0};

  if (argc < 2) {
    fprintf(stderr, "Usage: csvinfo [-s] files\n");
    exit(EXIT_FAILURE);
  }

  if (csv_init(&p, options) != 0) {
    fprintf(stderr, "Failed to initialize csv parser\n");
    exit(EXIT_FAILURE);
  }

  csv_set_space_func(p, is_space);
  csv_set_term_func(p, is_term);

  while (*(++argv)) {
    if (strcmp(*argv, "-s") == 0) {
      options = CSV_STRICT;
      csv_opts(p, options);
      continue;
    }

    fp = fopen(*argv, "rb");
    if (!fp) {
      fprintf(stderr, "Failed to open %s: %s\n", *argv, strerror(errno));
      continue;
    }

    while ((bytes_read=fread(buf, 1, 1024, fp)) > 0) {
      if (csv_parse(p, buf, bytes_read, cb1, cb2, &c) != bytes_read) {
        fprintf(stderr, "Error while parsing file: %s\n", csv_strerror(csv_error(p)));
      }
    }

    csv_fini(p, cb1, cb2, &c);

    if (ferror(fp)) {
      fprintf(stderr, "Error while reading file %s\n", *argv);
      fclose(fp);
      continue;
    }

    fclose(fp);
    printf("%s: %lu fields, %lu rows\n", *argv, c.fields, c.rows);
  }

  csv_free(p);
  exit(EXIT_SUCCESS);
}

==> libcsv-3.0.0.tar.gz: libcsv-3.0.0/examples/csvinfo.c <==
/*
csvinfo - reads CSV data from input file(s) and reports the number
          of fields and rows encountered in each file

Copyright (C) 2007  Robert Gamble

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include "libcsv/csv.h"

struct counts {
  long unsigned fields;
  long unsigned rows;
};

void cb1 (void *s, size_t len, void *data) { ((struct counts *)data)->fields++; }
void cb2 (int c, void *data) { ((struct counts *)data)->rows++; }

static int is_space(unsigned char c) {
  if (c == CSV_SPACE || c == CSV_TAB) return 1;
  return 0;
}

static int is_term(unsigned char c) {
  if (c == CSV_CR || c == CSV_LF) return 1;
  return 0;
}


int
main (int argc, char *argv[])
{
  FILE *fp;
  struct csv_parser p;
  char buf[1024];
  size_t bytes_read;
  unsigned char options = 0;
  struct counts c = {0, 0};

  if (argc < 2) {
    fprintf(stderr, "Usage: csvinfo [-s] files\n");
    exit(EXIT_FAILURE);
  }

  if (csv_init(&p, options) != 0) {
    fprintf(stderr, "Failed to initialize csv parser\n");
    exit(EXIT_FAILURE);
  }

  csv_set_space_func(&p, is_space);
  csv_set_term_func(&p, is_term);

  while (*(++argv)) {
    if (strcmp(*argv, "-s") == 0) {
      options = CSV_STRICT;
      csv_set_opts(&p, options);
      continue;
    }

    fp = fopen(*argv, "rb");
    if (!fp) {
      fprintf(stderr, "Failed to open %s: %s\n", *argv, strerror(errno));
      continue;
    }

    while ((bytes_read=fread(buf, 1, 1024, fp)) > 0) {
      if (csv_parse(&p, buf, bytes_read, cb1, cb2, &c) != bytes_read) {
        fprintf(stderr, "Error while parsing file: %s\n", csv_strerror(csv_error(&p)));
      }
    }

    csv_fini(&p, cb1, cb2, &c);

    if (ferror(fp)) {
      fprintf(stderr, "Error while reading file %s\n", *argv);
      fclose(fp);
      continue;
    }

    fclose(fp);
    printf("%s: %lu fields, %lu rows\n", *argv, c.fields, c.rows);
  }

  csv_free(&p);
  exit(EXIT_SUCCESS);
}
